#!/bin/bash
#########################################################
#							#
#		SNMP Printer Check			#
#							#
# check_snmp_printer.sh					#
# 	Version 1.5 (January 15, 2010)			#
#							#
# Authored by Jason Leonard				#
# 	E-mail: jason_leonard@yahoo.com			#
#							#
# Version History					#
#							#
#	1.5	It was brought to my attention that my	#
#	comparisons to critical and warning levels were	#
#	not INCLUSIVE of those levels. So, when toner	#
#	level was reaching 5 or 20, the plugin was be-	#
#	having oddly. This has been corrected now, and	#
#	the plugin uses a variable for WARNING and 	#
#	CRITICAL so the levels can be changed more	#
#	easily by the end user. I plan to implement	#
#	passing those levels to the plugin very soon.	#
#							#
#	1.4	After rewriting my string matching, it	# 
#	was brought to my attention that some consum-	#
#	ables are not always represented by a unique	#
#	string, so exact matching on a string (the 	#
#	original intended purpose of the CONSUM option)	#
#	was still needed. As a result, I added the new	#
#	CONSUMX option. Essentially, CONSUMX <string> 	#
#	looks for a consumable to have an EXACT match 	#
#	to the string passed to it. This allows the 	#
#	plugin to have both the original functionality	#
#	intended, and the added functionality of output	#
#	on all matches to a substring.			#
#							#
#	1.3	I again rewrote the string matching for	#
#	consumable checks. Now, it functions more like	#
#	the CONSUM ALL check, except it loops on all	#
#	matching strings. For example, on certain	#
#	copiers, "CONSUM Yellow" will now report values	#
#	for Yellow Toner, Yellow Imaging Unit and	#
#	Yellow Drum. In a more handy use, CONSUM Toner	#
#	would just report all toner levels, etc. This	#
#	was done after I had to acknowledge that there	#
#	is not always a unique string to match amongst	#
#	consumables.
#							#
#	1.2	Fixed a bug with handling the string	#
#	matching on individual consumable checks. I was	#
#	passing the string variable without "s, so the	#
#	script was not parsing the string properly.	#
#	Many thanks to Tobias Balle-Petersen for	#
#	bringing this to my attention.			#
#							#
#	1.1	Rewrote  paper tray handling function.	#
#	Now plugin uses 1 function instead of 3, with	#
#	code to handle any paper tray. The function	#
#	also calculates tray capacity as % to get more	#
#	accurate status codes. The old plugin only 	#
#	looked for a -3 value, but many printers do use	#
#	positive values to indicate number of pages,	#
#	and would return errors before. I also updated 	#
#	the MESSAGES option to be able to handle 	#
#	null returns from printers that don't use this	#
#	OID. Lastly, some code cleanup was in order 	#
#	with SNMP checks and similar items.		#
#							#
#	1.0	initial release				#
#							#
# Overview						#
#	This plugin is a rewrite of the SNMP printer	#
# check provided by Monitoring Solutions. In operating	#
# the plugin at our environment, I noticed the output	#
# was combined as one long paragraph when using	the 	#
# CONSUM ALL option (definitely a favorite of mine).	#
# For devices with numerous consumables (like copiers), #
# this made it difficult to quickly find the empty 	#
# consumable when an alert would come in. So I set out	#
# to spruce up the output of the plugin - using one 	#
# consumable per line.				 	#
#							#
#	In the process of working on this project, I 	#
# also realized the original plugin was more if/then 	#
# statements than I had seen since programming class 	#
# in college. So I set out to make the code a little 	#
# bit cleaner for faster execution. I also wanted to 	#
# add SNMP checks, as I found the original plugin 	#
# would return OK status to Nagios even if SNMP was	#
# broken, because the plugin assumes OK status and does	#
# no SNMP checking. I needed to get some experience	#
# with this, too, as my other plugins currently do not	#
# do SNMP pre-flights.					#
#							#
#	Lastly, I decided to rewrite the options and	#
# other pre-flight checks that are present in my other	#
# plugins. I like to be thorough with making sure the	#
# program won't return output if the input is garbage!	#
#							#
# NOTE:							#
#	Because CONSUM ALL uses a multi-line output, 	#
# you will need to use the $LONGSERVICEOUTPUT$ macro 	#
# in your service notification commands!		#
#							#
#	Like the original, this plugin is distributed	#
# under the GNU GPL license. You may re-destribute only #
# according to the terms of the GNU GPL.		#
#							#
#########################################################
#							#
# ----- ORIGINAL SCRIPT COMMENTS PROVIDED BELOW -----	#
#							#
#########################################################
# +-------------------------------------------------------------------------+
# | Copyright (C) 2007 S&L Netzwerktechnik                 |
# |                                     |
# | This program is free software; you can redistribute it and/or      |
# | modify it under the terms of the GNU General Public License       |
# | as published by the Free Software Foundation; either version 2     |
# | of the License, or (at your option) any later version.         |
# |                                     |
# | This program is distributed in the hope that it will be useful,     |
# | but WITHOUT ANY WARRANTY; without even the implied warranty of     |
# | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the      |
# | GNU General Public License for more details.              |
# +-------------------------------------------------------------------------+
# |Monitoring Solution: The Complete Network Monitoring Solution :)     |
# +-------------------------------------------------------------------------+
# | This code is designed, written, and maintained by the Serge Mueller/ S&L|
# +-------------------------------------------------------------------------+
# | http://www.monitoring-solution.de/                   |
# +-------------------------------------------------------------------------+
#########################################################
#
# check_snmp_printer - printer/consumables monitoring plugin for Nagios
# Version: 0.1.0
#
#########################################################
#
# CHANGE LOG
#
# 07-05-23	Script creation
# 2007-06-06	Added CONSUM ALL functionality
#
#########################################################
#
# COMMENTS
# The Papertray stuff may need some attention...
# Most printers' output sucks there
#
#########################################################
#
# DEPENDS On
# snmpwalk - if needet change variable below
# snmpget - see above
#
#########################################################


#########################################################
###		print_help Function		   ###
#########################################################

function print_help () {

cat<<EOU

Usage of $APPNAME
---------------------------------------------------------------------

Usable Options:
	<hostname>
		 The IP address or hostname of the system to check
		 (required option)

	<community>
		 The SNMP Community variable - use the name of your SNMP community with read privileges
		 (required option)


Examples:
	$APPNAME <hostname> <community> CHECK_TO_PERFORM

		where CHECK_TO_PERFORM can be:
			MESSAGES
			MODEL
			CONSUM TEST
			CONSUM <string>
			CONSUM ALL
			CONSUMX <string>
			TRAY <number>
			PAGECOUNT

	CONSUM TEST will give you the exact names of installed consumables

		For example,
			Black Toner Cartridge HP C4191A

		For monitoring this consumable, you can then call the function as so:
			$APPNAME <hostname> <community> CONSUM Black

	CONSUM <string> will give you all consumables matching the string that is passed
	
		For example, 
			CONSUM Toner 
			
		will only show you all toner levels (no staple cartridges or imaging units)

	CONSUM ALL will give you all consumable output at once.

	CONSUMX <string> will give you results only for the consumable that ***EXACTLY*** matches the  string passed
	
		For example, 
			CONSUMX "Black Toner Cartridge" 
		
		will only give you the usage for a consumable named "Black Toner Cartridge". It will not give you results for "Black Toner Cartridge 1" or "Black Toner".

---------------------------------------------------------------------

EOU
exit 3
}


#########################################################
###		check_model function		   ###
#########################################################

function check_model(){
#	Vendor specific items to code here!
#		possibly serial #
	MODEL=`snmpget -v1 -c $COMMUNITY $HOSTNAME 1.3.6.1.2.1.25.3.2.1.3.1 2>/dev/null | cut -d " " -f4-`
	SERIAL=`snmpwalk -v1 -c $COMMUNITY $HOSTNAME 1.3.6.1.2.1.43.5.1.1.17 2>/dev/null | cut -d " " -f4- | tr -d "\""`
	
	printf "$MODEL, Serial # $SERIAL"
	exit 0
}



#########################################################
###		check_messages function		   ###
#########################################################

function check_messages(){
#	Vendor specific items to code here!
	MESSAGES=`snmpwalk -v1 -c $COMMUNITY $HOSTNAME 1.3.6.1.2.1.43.18.1.1.8 2>/dev/null | gawk -F: '{ print $4 }' | tr -d "\"" | tr "\n" "\!"`
	
	if [ -z "$MESSAGES" ]; then
		printf "(Can't determine messages - printer does not support alerts using this OID"
#		exit 3
	else
		printf "$MESSAGES"
	fi
	
	exit 0

}



#########################################################
###		check_page_count function	   ###
#########################################################

function check_page_count(){
	PAGE_COUNT=`snmpget -v1 -c $COMMUNITY $HOSTNAME 1.3.6.1.2.1.43.10.2.1.4.1.1 2>/dev/null | cut -d " " -f4- | sed -e :x -e 's/\([0-9][0-9]*\)\([0-9][0-9][0-9]\)/\1,\2/' -e 'tx'`
	
	printf "Pagecount is $PAGE_COUNT|Pages=$PAGE_COUNT;;;"
	exit 0
}



#########################################################
###		check_one_consumable function	   ###
#########################################################

function check_one_consumable(){

	MARKERS=`snmpwalk -v1 -c $COMMUNITY $HOSTNAME 1.3.6.1.2.1.43.11.1.1.6.1 2>/dev/null`
	
	# We're looking for the text in the snmpwalk to match the exact string passed to us
	MARKER_NAME=$(echo "$MARKERS" | cut -d " " -f4- | tr -d "\"" | egrep -x "$1")

	if [ -z "$MARKER_NAME" ]; then 
		printf "UNKNOWN - OID not found! Your printer may not support checking this consumable."
		printf "Use the CONSUM TEST option to determine which consumables may be monitored."
		exit 3 
	fi

	# Get the exact ID for the marker we are evaluating
	MARKER_ID=$(echo "$MARKERS" | egrep -i "$1" | cut -d "=" -f1 | cut -d "." -f8)

	# Read in our capacities from SNMP
	CURRENT_CAPACITY=`snmpget -v1 -c $COMMUNITY $HOSTNAME 1.3.6.1.2.1.43.11.1.1.9.1.$MARKER_ID 2>/dev/null | cut -d " " -f4- | tr -d "\""`
	MAX_CAPACITY=`snmpget -v1 -c $COMMUNITY $HOSTNAME 1.3.6.1.2.1.43.11.1.1.8.1.$MARKER_ID 2>/dev/null | cut -d " " -f4- | tr -d "\""`

	if [ "$MAX_CAPACITY" -gt 0 ] && [ "$CURRENT_CAPACITY" -gt 0 ]; then 
	# if the component has a measurable status, let's report it's status code and let user know the % usage
		let "CURRENT_CAPACITY= $CURRENT_CAPACITY * 100 / $MAX_CAPACITY"
		if [ "$CURRENT_CAPACITY" -gt "$WARNING" ]; then 
			printf "$MARKER_NAME is at $CURRENT_CAPACITY%% - OK!\n | $MARKER_NAME=$CURRENT_CAPACITY;;$MAX_CAPACITY;"
			exit 0
		else 
			if [ "$CURRENT_CAPACITY" -le "$WARNING" ] && [ "$CURRENT_CAPACITY" -gt "$CRITICAL" ]; then
				printf "$MARKER_NAME is at $CURRENT_CAPACITY%% - WARNING!\n | $MARKER_NAME=$CURRENT_CAPACITY;;$MAX_CAPACITY;"
				exit 1 
			else 
				if [ "$CURRENT_CAPACITY" -le "$CRITICAL" ]; then
					printf "$MARKER_NAME is at $CURRENT_CAPACITY%% - CRITICAL!\n | $MARKER_NAME=$CURRENT_CAPACITY;;$MAX_CAPACITY;"
					exit 2
				fi
			fi
		fi

	else
		# the component does not have a measurable status indication
		case "$CURRENT_CAPACITY" in
			"-3") # A value of (-3) means that the printer knows that there is some supply/remaining space
				printf "$MARKER_NAME is OK!\n"
				exit 0
			;;
			"-2") # The value (-2) means unknown
				printf "$MARKER_NAME is at WARNING level!\n"
				exit 1
			;;
			"0") # Something is empty!
				printf "$MARKER_NAME is at CRITICAL level!\n"
				exit 2
			;;
		esac
	fi

}

#########################################################
###		check_consumables function	   ###
#########################################################

function check_consumables(){

	case "$1" in
		"TEST")	# User passed "TEST" parameter - output what consumables are available
			printf "Consumables you may monitor:\n"
			snmpwalk -v1 -c $COMMUNITY $HOSTNAME 1.3.6.1.2.1.43.11.1.1.6.1 2>/dev/null | cut -d " " -f4-  
			exit 0
		;;
		"ALL") # User passed ALL parameter - check everything!
			EXITCODE=0
			EXITSTRING=""
			PERFDAT=""

			for MARKER_ID in `snmpwalk -v1 -c $COMMUNITY $HOSTNAME 1.3.6.1.2.1.43.11.1.1.6.1 2>/dev/null | egrep -oe '[[:digit:]]+\ =' | cut -d " " -f1`
			do
				MARKER_NAME=`snmpget -v1 -c $COMMUNITY $HOSTNAME 1.3.6.1.2.1.43.11.1.1.6.1.$MARKER_ID 2>/dev/null | cut -d " " -f4- | tr -d "\""`
 
				# If name is blank, then we have an issue - the printer has a device listed that it can not check through SNMP
				if [ -z "$MARKER_NAME" ]; then
					printf "UNKNOWN - OID not found! Your printer may not support checking this consumable. "
					printf " Use the CONSUM TEST option to determine which consumables may be monitored."
					exit 3
				fi
				
				CURRENT_CAPACITY=`snmpget -v1 -c $COMMUNITY $HOSTNAME 1.3.6.1.2.1.43.11.1.1.9.1.$MARKER_ID 2>/dev/null | cut -d " " -f4- | tr -d "\""`
				MAX_CAPACITY=`snmpget -v1 -c $COMMUNITY $HOSTNAME 1.3.6.1.2.1.43.11.1.1.8.1.$MARKER_ID 2>/dev/null | cut -d " " -f4- | tr -d "\""`

				# If our object has a measurable (positive) CURRENT_CAPACITY, such as toner level
				#	Let's report the status as a %
				if [ "$MAX_CAPACITY" -gt 0 ] && [ "$CURRENT_CAPACITY" -gt 0 ]; then
					let "CURRENT_CAPACITY=$CURRENT_CAPACITY * 100 / $MAX_CAPACITY"
					
					if [ "$CURRENT_CAPACITY" -gt "$WARNING" ]; then
						EXITSTRING="$EXITSTRING$MARKER_NAME is at $CURRENT_CAPACITY%% - OK!\n"
						PERFDAT="$PERFDAT $MARKER_NAME=$CURRENT_CAPACITY;;$MAX_CAPACITY;"
					else
						if [ "$CURRENT_CAPACITY" -le "$WARNING" ] && [ "$CURRENT_CAPACITY" -gt "$CRITICAL" ];	then
							EXITSTRING="$EXITSTRING$MARKER_NAME is at $CURRENT_CAPACITY%% - WARNING!\n"
							PERFDAT="$PERFDAT $MARKER_NAME=$CURRENT_CAPACITY;;$MAX_CAPACITY;"
							if [ "$EXITCODE" -lt 1 ]; then
								EXITCODE=1
							fi
						else
							if [ "$CURRENT_CAPACITY" -le "$CRITICAL" ]; then
								EXITSTRING="$EXITSTRING$MARKER_NAME is at $CURRENT_CAPACITY%% - CRITICAL!\n"
								PERFDAT="$PERFDAT $MARKER_NAME=$CURRENT_CAPACITY;;$MAX_CAPACITY;"
								EXITCODE=2
							fi
						fi
					fi

				else
				
					# Our object is not measurable - it's either FULL or EMPTY (such as a punch dust box)
					#	Let's report on it's status using appropriate terminology
					case "$CURRENT_CAPACITY" in
						"-3") # A value of (-3) means that the printer knows that there is some supply/remaining space
							EXITSTRING="$EXITSTRING$MARKER_NAME is FULL - OK!\n"
						;;
						"-2") # The value (-2) means unknown
							EXITSTRING="$EXITSTRING$MARKER_NAME is at WARNING level!\n"
							if [ "$EXITCODE" -lt 1 ]; then
								EXITCODE=1
							fi
						;;
						"0") # Something is empty!
							EXITSTRING="$EXITSTRING$MARKER_NAME is at CRITICAL level!\n"
							EXITCODE=2
						;;
					esac
				
				fi

			done
		
			printf "$EXITSTRING|$PERFDAT"
			exit $EXITCODE
		;;
		*) # A string was passed, on which we will match to available consumable strings
			EXITCODE=0
			EXITSTRING=""
			PERFDAT=""

			MARKERS_MATCHED=`snmpwalk -v1 -c $COMMUNITY $HOSTNAME 1.3.6.1.2.1.43.11.1.1.6.1 2>/dev/null | egrep -i "$1"`
			NUM_MARKERS_MATCHED=$(echo "$MARKERS_MATCHED" | wc -l)

			# Get IDs and names of all consumables matching the string
			MARKER_IDS=$(echo "$MARKERS_MATCHED" | cut -d "=" -f1 | cut -d "." -f8)

			# Let's loop through each marker that matched our string
			CURRENT_MARKER=1
			
			while [ $CURRENT_MARKER -le $NUM_MARKERS_MATCHED ]; do
				# Consumable IDs are not just 1-9, they can vary - so let's be careful to get the 1st ID of the matching markers
				CURRENT_MARKER_ID=$(echo "$MARKER_IDS" | awk NR==$CURRENT_MARKER)
				CURRENT_MARKER_NAME=`snmpget -v1 -c $COMMUNITY $HOSTNAME 1.3.6.1.2.1.43.11.1.1.6.1.$CURRENT_MARKER_ID 2>/dev/null | cut -d " " -f4- | tr -d "\""`
				
				if [ -z "$CURRENT_MARKER_NAME" ]; then 
					printf "UNKNOWN - OID not found! Your printer may not support checking this consumable. "
					printf "Use the CONSUM TEST option to determine which consumables may be monitored."
					exit 3 
				fi

				# Determine capacities for the current marker
				CURRENT_CAPACITY=`snmpget -v1 -c $COMMUNITY $HOSTNAME 1.3.6.1.2.1.43.11.1.1.9.1.$CURRENT_MARKER_ID 2>/dev/null | cut -d " " -f4- |tr -d "\""`
				MAX_CAPACITY=`snmpget -v1 -c $COMMUNITY $HOSTNAME 1.3.6.1.2.1.43.11.1.1.8.1.$CURRENT_MARKER_ID 2>/dev/null | cut -d " " -f4- | tr -d "\""`
				
				if [ "$MAX_CAPACITY" -gt 0 ] && [ "$CURRENT_CAPACITY" -gt 0 ]; then 
				# if the component has a measurable status, let's report it's status code and let user know the % usage
					let "CURRENT_CAPACITY=$CURRENT_CAPACITY * 100 / $MAX_CAPACITY"
					if [ "$CURRENT_CAPACITY" -gt "$WARNING" ]; then 
						EXITSTRING="$EXITSTRING$CURRENT_MARKER_NAME is at $CURRENT_CAPACITY%% - OK!\n"
						PERFDAT="$PERFDAT $CURRENT_MARKER_NAME=$CURRENT_CAPACITY;;$MAX_CAPACITY;"
					else 
						if [ "$CURRENT_CAPACITY" -le "$WARNING" ] && [ "$CURRENT_CAPACITY" -gt "$CRITICAL" ]; then
							EXITSTRING="$EXITSTRING$CURRENT_MARKER_NAME is at $CURRENT_CAPACITY%% - WARNING!\n"
							PERFDAT="$PERFDAT $CURRENT_MARKER_NAME=$CURRENT_CAPACITY;;$MAX_CAPACITY;"
							if [ "$EXITCODE" -lt 1 ]; then
								EXITCODE=1
							fi
						else 
							if [ "$CURRENT_CAPACITY" -le "$CRITICAL" ]; then
								EXITSTRING="$EXITSTRING$CURRENT_MARKER_NAME is at $CURRENT_CAPACITY%% - CRITICAL!\n"
								PERFDAT="$PERFDAT $CURRENT_MARKER_NAME=$CURRENT_CAPACITY;;$MAX_CAPACITY;"
								EXITCODE=2
							fi
						fi
					fi

				else
					# the component does not have a measurable status indication
					case "$CURRENT_CAPACITY" in
						"-3") # A value of (-3) means that the printer knows that there is some supply/remaining space
							EXITSTRING="$EXITSTRING$CURRENT_MARKER_NAME is OK!\n"
							EXITCODE=0
						;;
						"-2") # The value (-2) means unknown
							EXITSTRING="$EXITSTRING$CURRENT_MARKER_NAME is at WARNING level!\n"
							EXITCODE=1
						;;
						"0") # Something is empty!
							EXITSTRING="$EXITSTRING$CURRENT_MARKER_NAME is at CRITICAL level!\n"
							EXITCODE=2
						;;
					esac
				fi
				
				# Counter increment
				CURRENT_MARKER=$[ $CURRENT_MARKER+1 ]
				
			done

			printf "$EXITSTRING|$PERFDAT"
			exit $EXITCODE
		;;
	esac
}


#########################################################
###		check_paper_trays Function	   ###
#########################################################

function check_paper_trays (){

#	Vendor specific items to code
#		tray names

	# Need to check that tray is supported first, then check info. on it...if problem, report on what CAN be monitored
	TRAY_STATUS=`snmpwalk -v1 -c $COMMUNITY $HOSTNAME 1.3.6.1.2.1.43.8.2.1.10.1.$1 2>/dev/null`
	
	if [ -z "$TRAY_STATUS" ]; then
		printf "UNKNOWN - OID not found! Your printer may not support checking this tray. Trays supported are " 
		snmpwalk -v1 -c $COMMUNITY $HOSTNAME 1.3.6.1.2.1.43.8.2.1.10.1 2>/dev/null | egrep -oe '[[:digit:]]+\ =' | cut -d " " -f1 | tr "\n" " "
		exit 3
	fi

	# Determine information about the tray
	TRAY_CAPACITY=$(echo "$TRAY_STATUS" | cut -d " " -f4- | tr -d "\"")
	TRAY_NAME=`snmpwalk -v1 -c $COMMUNITY $HOSTNAME 1.3.6.1.2.1.43.8.2.1.13.1.$1 2>/dev/null | cut -d " " -f4- | tr -d "\""`
	
	# Determine paper size of current tray
	TRAY_FEED_DIMENSION=`snmpwalk -v1 -c $COMMUNITY $HOSTNAME 1.3.6.1.2.1.43.8.2.1.4.1.$1 2>/dev/null | cut -d " " -f4- | tr -d "\""`
	TRAY_FEED_DIMENSION_UNITS=`snmpwalk -v1 -c $COMMUNITY $HOSTNAME 1.3.6.1.2.1.43.8.2.1.2.1.$1 2>/dev/null | cut -d " " -f4- | tr -d "\""`
	TRAY_XFEED_DIMENSION=`snmpwalk -v1 -c $COMMUNITY $HOSTNAME 1.3.6.1.2.1.43.8.2.1.5.1.$1 2>/dev/null | cut -d " " -f4- | tr -d "\""`
	TRAY_XFEED_DIMENSION_UNITS=`snmpwalk -v1 -c $COMMUNITY $HOSTNAME 1.3.6.1.2.1.43.8.2.1.3.1.$1 2>/dev/null | cut -d " " -f4- | tr -d "\""`

	case "$TRAY_FEED_DIMENSION_UNITS" in
		"3") # convert ten thousandths of an inch to inches
		TRAY_FEED_DIMENSION=$(echo "$TRAY_FEED_DIMENSION"*10000 | bc)
		;;
		"4") # convert micrometers to inches, and get the int portion
		TRAY_FEED_DIMENSION=$(echo "$TRAY_FEED_DIMENSION"*0.0000393700787 | bc)
		TRAY_FEED_DIMENSION=$(echo "$TRAY_FEED_DIMENSION"+0.5 | bc)
		TRAY_FEED_DIMENSION=$(echo "$TRAY_FEED_DIMENSION"/1 | bc)
		;;
	esac

	case "$TRAY_XFEED_DIMENSION_UNITS" in
		"3") # convert ten thousandths of an inch to inches
		TRAY_XFEED_DIMENSION=$(echo "$TRAY_XFEED_DIMENSION"*10000 | bc)
		;;
		"4") # convert micrometers to inches, and get the int portion
		TRAY_XFEED_DIMENSION=$(echo "$TRAY_XFEED_DIMENSION"*0.0000393700787 | bc)
		TRAY_XFEED_DIMENSION=$(echo "$TRAY_XFEED_DIMENSION"+0.01 | bc)
		TRAY_XFEED_DIMENSION=$(echo "scale=1; $TRAY_XFEED_DIMENSION"/1 | bc)
		;;
	esac
	
	# Some manufacturers do not set the tray name OID, so we'll assume a generic name depending on the # passed to the function
	if [ -z "$TRAY_NAME" ]; then
		TRAY_NAME="Tray $1"
	fi
	
	case "$TRAY_CAPACITY" in
		"-3") # The value (-3) means that the printer knows that at least one unit remains.
			printf "$TRAY_NAME is OK!"
			exit 0
		;;
		"-2") # The value (-2) means unknown
			printf "$TRAY_NAME status is UNKNOWN!"
			exit 3
		;;
		"0") # 0 means there is no paper left! This is our only critical value.
			printf "$TRAY_NAME is at CRITICAL level! Please refill with more $TRAY_XFEED_DIMENSIONx$TRAY_FEED_DIMENSION paper"
			exit 2
		;;
		*) # A positive number indicates how many pages are left. We'll calculate what % of capacity this is and determine status
			TRAY_MAX_CAPACITY=`snmpget -v1 -c $COMMUNITY $HOSTNAME 1.3.6.1.2.1.43.8.2.1.9.1.$1 2>/dev/null | cut -d " " -f4- | tr -d "\""`
			let "TRAY_CAPACITY = $TRAY_CAPACITY * 100 / $TRAY_MAX_CAPACITY"
				
			if [ "$TRAY_CAPACITY" -gt "$CRITICAL" ]; then
				printf "$TRAY_NAME is at $TRAY_CAPACITY%% - OK!\n"
				exit 0
			else
				if [ "$TRAY_CAPACITY" -le "$CRITICAL" ]; then
					printf "$TRAY_NAME is at $TRAY_CAPACITY%% - WARNING! Please refill with more $TRAY_XFEED_DIMENSION x $TRAY_FEED_DIMENSION paper\n"
					exit 1
				fi
			fi
		;;
	esac
	
}


#########################################################
###			MAIN CODE		   ###
#########################################################

APPNAME=$(basename $0)
HOSTNAME=$1
COMMUNITY=$2

# Set a warning at 20% of consumable
WARNING="20"
# Set a critical at 5% of consumable. The standard 10 seems to be high for
#	most consumables, which move very slowly unless you have a very high volume.
#	I find this typical of toner, maintenance kits - you name it!
CRITICAL="5"

# Check that all required binaries to use snmpwalk are available first
#	EXIT with an UNKNOWN status if not
binaries=snmpwalk

for required_binary in $binaries; do
	which $required_binary > /dev/null
	if [ "$?" != '0' ];then
		printf "UNKNOWN: $APPNAME: No usable '$required_binary' binary in '$PATH'\n"
		exit 3
	fi
done


#Check for server response - is SNMP even setup okay?
snmpwalk -Os -c $COMMUNITY -v1 $HOSTNAME 1.3.6.1.2.1.43.11.1.1.6.1 > /dev/null 2>&1
if [ $? -ne 0 ]; then
	echo "SYSTEM WARNING - no SNMP response from $HOSTNAME! Make sure SNMP is configured properly."
	exit 1
fi


# If there are less than 3 parameters, the user is missing something necessary
#	(all parameters are required)
if [ "$#" -lt 3 ]; then
	# Output the usage for the user
	print_help
	exit 3
fi		


# Check the 3rd parameter passed to the program; it represents the check the user wants to run
case "$3" in
	"MESSAGES") 
		check_messages;;
	"MODEL") 
		check_model;;
	"CONSUM") 
		check_consumables "$4";;
	"CONSUMX") 
		check_one_consumable "$4";;
	"TRAY") 
		check_paper_trays "$4";;
	"PAGECOUNT") 
		check_page_count;;
	"TEST") 
		check_consumables "$3";;
	*) 
		print_help
		exit 3;;
esac


# Theoretically, this is a catch for any unexpected results
printf "UNKNOWN - something went wrong! Please check all function parameters."
exit 3
