#!/usr/bin/env python
#
# Copyright (C) 2010  James Troup <james.troup@canonical.com>
"""Check Asterik ZAP channels."""

import optparse
import nagios_plugin

################################################################################
# Example input file:

#   Chan Extension  Context         Language   MOH Interpret
# pseudo            from-pstn                  default
#      1            from-pstn                  default
#      2            from-pstn                  default
#      3            from-pstn                  default
#      4            from-pstn                  default
#      5            from-pstn                  default
#      6            from-pstn                  default
#      7            from-pstn                  default
#      8            from-pstn                  default
#      9            from-pstn                  default
#     10            from-pstn                  default
#     11            from-pstn                  default
#     12            from-pstn                  default
#     13            from-pstn                  default
#     14            from-pstn                  default
#     15            from-pstn                  default

###########################################################################

def check_asterisk_zap():
    """Check Asterisk ZAP channels.

Expects the output from:

  asterisk -r -x 'zap show channels'

in a file and that file is fresh (staleness is configurable).

Checks that:

 a) there is a pseudo zap channel

 b) the expected number of (non-pseudo) zap channels are present

 c) usage of the channels is not above configurable warning and
    critical levels
"""

    parser = optparse.OptionParser()
    parser.add_option("-a", "--age", dest="file_age", default=1200)
    parser.add_option("-f", "--filename", dest="filename",
                      default="/var/lib/nagios/cat-zap-channels.txt")
    parser.add_option("-l", "--lines", dest="num_lines", default=5555)
    parser.add_option("-w", "--warning", dest="congestion_warn", default=60)
    parser.add_option("-c", "--critical", dest="congestion_crit", default=80)
    (opts, _) = parser.parse_args()

    nagios_plugin.check_file_freshness(opts.filename, opts.file_age)
    input_file = open(opts.filename, 'r')

    header_line = input_file.readline().rstrip()
    expected_header = "   Chan Extension  Context         Language   MOH Interpret"
    if header_line != expected_header:
        raise nagios_plugin.UnknownError("Unrecognised header line in 'zap show channels' output")

    lines_in_use = 0
    have_pseudo = False
    num_lines = 0
    for line in input_file.readlines():
        line = line.rstrip()
        if not line:
            continue

        # Check channel
        channel = line[:8]
        if channel.strip() == "pseudo":
            have_pseudo = True
        else:
            num_lines += 1

        # If we can see an extension for the channel then the line is in use
        extension = line[8:19]
        if extension.strip():
            lines_in_use += 1

    # Check the sanity of the channels
    if not have_pseudo:
        raise nagios_plugin.CriticalError("pseudo zap channel not found")
    if num_lines != int(opts.num_lines):
        raise nagios_plugin.CriticalError("expected %s lines, but only found %s" \
                                          % (opts.num_lines, num_lines))
    if num_lines < lines_in_use:
        raise nagios_plugin.CriticalError("YOU'RE DOING IT WRONG. %s lines, %s in use" \
                                          % (num_lines, lines_in_use))

    if int(opts.num_lines) > 0:
        # Check the usage of lines
        usage = (lines_in_use / float(num_lines)) * 100
        usage_str = "%.0f%% (%s of %s) of lines used" % (usage, lines_in_use,
                                                         num_lines)
        if usage >= int(opts.congestion_crit):
            raise nagios_plugin.CriticalError(usage_str)
        if usage >= int(opts.congestion_warn):
            raise nagios_plugin.WarnError(usage_str)
        print "All OK: %s" % (usage_str)
    else:
        print "All OK: [pseudo only]"

################################################################################

def main():
    nagios_plugin.try_check(check_asterisk_zap)

################################################################################

if __name__ == "__main__":
    main()
