#!/usr/bin/python
"""check_couchdb_alive - checks whether CouchDB is alive"""

# Roman Yepishev <roman.yepishev@canonical.com>
# Liam Young: Changed name from check_couchdb2.py to check_couchdb_alive.py

from optparse import OptionParser
from oauth import oauth
import httplib2, simplejson, urlparse, cgi, urllib, sys
import uuid
import logging

class ApiException(Exception):
    """Occurs when /api/ misbehaves"""
    pass

class CouchDbException(Exception):
    """Occurs when couchd reply is invalid"""
    pass

TEST_VIEW = """{
    "_id": "_design/test",
    "language": "javascript",
    "views": {
        "test": {
            "map": "function(doc) { emit(doc.id, doc['message']); }"
        }
    }
}"""
TEST_DOCUMENT = """{
    "message": "hello"
}"""

class OAuthHttpClient(object):

    def __init__(self):
        self.signature_method = oauth.OAuthSignatureMethod_HMAC_SHA1()
        self.consumer = None
        self.token = None
        self.use_oauth = True
        self.client = httplib2.Http()

    def set_consumer(self, consumer_key, consumer_secret):
        self.use_oauth = True
        self.consumer = oauth.OAuthConsumer(consumer_key,
                                            consumer_secret)

    def set_token(self, token, token_secret):
        self.use_oauth = True
        self.token = oauth.OAuthToken( token, token_secret)

    def set_basic_auth(self, username, password):
        self.use_oauth = False
        self.client.add_credentials(username, password)

    def _get_oauth_request_header(self, url, method):
        """Get an oauth request header given the token and the url"""
        query = urlparse.urlparse(url).query

        oauth_request = oauth.OAuthRequest.from_consumer_and_token(
            http_url=url,
            http_method=method,
            oauth_consumer=self.consumer,
            token=self.token,
            parameters=dict(cgi.parse_qsl(query))
        )
        oauth_request.sign_request(oauth.OAuthSignatureMethod_HMAC_SHA1(),
                                   self.consumer, self.token)
        return oauth_request.to_header()

    def request(self, url, method="GET", body=None, headers={}):
        if self.use_oauth:
            oauth_header = self._get_oauth_request_header(url, method)
            headers.update(oauth_header)

        return self.client.request(url, method, headers=headers, body=body)

class Application(object):

    def __init__(self):
        self.token = None
        self.consumer = None
        self.couchdb_host = None
        self.couchdb_dbpath = None
        self.client = OAuthHttpClient()
        self.api = None

    def get_token_from_options(self, options):
        if not options.oauth:
            return False
        items = options.oauth.split(':')

        if len(items) == 2:
            consumer_key = 'ubuntuone'
            consumer_secret = 'hammertime'
            token_key, token_secret = items
        elif len(items) == 4:
            consumer_key, consumer_secret, token_key, token_secret = items
        else:
            return False

        self.consumer = oauth.OAuthConsumer(consumer_key, consumer_secret)
        self.token = oauth.OAuthToken(token_key, token_secret)

        return True

    def run(self, options):
        """Entry point for the application"""

        has_auth = False
        if options.oauth:
            has_auth = self.get_token_from_options(options)
            self.client.set_consumer(self.consumer.key, self.consumer.secret)
            self.client.set_token(self.token.key, self.token.secret)
        elif options.auth:
            has_auth = True
            user, password = options.auth.split(':')
            self.client.set_basic_auth(user, password)

        if not has_auth:
            print "ERROR: Access token cannot be found. I tried\n"\
                "options and found nothing\n"\
                "\n"\
                "Please get an OAuth or Basic auth tokens.\n"
            raise Exception("No authentication token")

        self.dbname = options.dbname
        self.api = options.api

        self.main(options)

    def main(self, options):
        if options.couchdb_host:
            self.couchdb_host = options.couchdb_host
        if options.couchdb_dbpath:
            self.couchdb_dbpath = options.couchdb_dbpath

        self.get_couchdb_info()

        # couchdb_dbpath must have all internal slashes escaped
        if self.couchdb_dbpath:
            couchdb_path = urllib.quote(self.couchdb_dbpath + '/', safe="")
        else:
            couchdb_path = ''
        couch_url = self.couchdb_host + '/' + couchdb_path + self.dbname

        # 1. verify that test database exists
        resp, content = self.client.request(couch_url)
        if resp['status'] == '404':
            # ok, need to create
            logger.debug("Database %s not found, creating.", self.dbname)
            resp, content = self.client.request(couch_url, 'PUT')
            if not resp['status'].startswith('2'):
                raise CouchDbException("Invalid response for database PUT")

        # 2. Check that test view exists
        design_url = couch_url + '/_design/test'
        resp, content = self.client.request(design_url)
        if resp['status'] == '404':
            # ok, need to create
            logger.debug("Test view not found, creating.")
            resp, content = self.client.request(design_url, 'PUT', body=TEST_VIEW)
            if not resp['status'].startswith('2'):
                logger.debug(content)
                raise CouchDbException("Invalid response for view PUT")

        # 3. Put document
        document_id = str(uuid.uuid1())
        document_url = couch_url + '/' + document_id
        resp, content = self.client.request(document_url, "PUT", body=TEST_DOCUMENT,
                headers={'content-type': 'application/json'})
        if not resp['status'].startswith('2'):
            logger.debug(content)
            raise CouchDbException("Invalid response for document PUT")

        info = simplejson.loads(content)
        document_rev = info['rev']

        # now execute view
        view_url = couch_url + '_design/test/_view/test'
        resp, content = self.client.request(document_url)
        if not resp['status'].startswith('2'):
            logger.debug(content)
            raise CouchDbException("Invalid response for view GET")

        # and now delete the document
        resp, content = self.client.request(
                document_url + '?rev=' + document_rev, 'DELETE')
        if not resp['status'].startswith('2'):
            logger.debug(content)
            raise CouchDbException("Invalid response for document DELETE")

    def get_couchdb_info(self):
        if self.couchdb_host is not None:
            return self.couchdb_host, self.couchdb_dbpath

        infourl = "%s/api/account/" % (self.api,)

        resp, content = self.client.request(infourl, "GET")
        if resp['status'] == "200":
            document = simplejson.loads(content)
        else:
            print content
            raise ValueError("Error retrieving user data (%s) from %s" % (
                            resp['status'], infourl))

        self.couchdb_host = document['couchdb']['host']
        self.couchdb_dbpath = document['couchdb']['dbpath']

        return self.couchdb_host, self.couchdb_dbpath

def setup_logging():
    """Sets up logging to stderr"""
    root_formatter = logging.Formatter(
            fmt="%(asctime)s:%(msecs)s %(name)s %(message)s")

    root_handler = logging.StreamHandler(sys.stderr)
    root_handler.setFormatter(root_formatter)

    instance = logging.getLogger('couchdb.client')
    instance.propagate = False
    instance.addHandler(root_handler)
    return instance

logger = setup_logging()

if __name__ == "__main__":
    parser = OptionParser(usage="prog [options]")
    parser.add_option("--oauth-signature-method", dest="sigmeth",
                      default="HMAC_SHA1",
                      help="OAuth signature method to use (PLAINTEXT or "
                      "HMAC_SHA1)")
    parser.add_option("--api", dest="api",
                      default="https://one.ubuntu.com", 
                      help="schema://host to use during /api/account call")
    parser.add_option("--couchdb-host", dest="couchdb_host",
                      default=None,
                      help="CouchDB host to use (instead of api calls)")
    parser.add_option("--couchdb-path", dest="couchdb_dbpath",
                      default=None,
                      help="User path (e.g. abc/def/50255 )")
    parser.add_option("--oauth", dest="oauth",
                      default=None,
                      help="Use the specified OAuth tokens")
    parser.add_option("--auth", dest="auth",
                      default=None,
                      help="Use the specified Basic authorization")
    parser.add_option("--dbname", dest="dbname",
                      help="Database Name for tests",
                      default="test")
    parser.add_option("--debug", dest="debug",
                      help="Output debug informatin",
                      default=False, action="store_true")

    (options, args) = parser.parse_args()

    if options.debug:
        logger.setLevel(logging.DEBUG)

    application = Application()

    try:
        application.run(options)
    except ApiException, e:
        print "COUCHDB: API exception: %s" % (e)
        sys.exit(2)
    except CouchDbException, e:
        print "COUCHDB: Could not create/delete database: %s" % (e)
        sys.exit(2)

    print "COUCHDB: Service is OK"
    sys.exit(0)

