#!/usr/bin/env python

# Copyright (C) 2008,2009  James Troup <james.troup@canonical.com>

################################################################################

import sys
import re
import datetime

import nagios_plugin

###########################################################################

def check_and_open(filename):
    nagios_plugin.check_file_freshness(filename, 20*60)
    return open(filename)

###########################################################################

### Temperature [hplog -t]
#           10        20        30        40        50        60        70
# 01234567890123456789012345678901234567890123456789012345678901234567890
# ID     TYPE        LOCATION      STATUS    CURRENT  THRESHOLD 
#  1  Basic Sensor I/O Zone        Normal   102F/ 39C 158F/ 70C 
#  2  Basic Sensor Ambient         Normal    73F/ 23C 102F/ 39C 
#  3  Basic Sensor CPU (1)         Normal    95F/ 35C 260F/127C 
#  4  Basic Sensor CPU (1)         Normal    95F/ 35C 260F/127C 
#  5  Basic Sensor Pwr. Supply Bay Normal   111F/ 44C 170F/ 77C 
#  6  Basic Sensor CPU (2)         Normal   ---F/---C 260F/127C 
#  7  Basic Sensor CPU (2)         Normal   ---F/---C 260F/127C 
# 
def check_temperature(input_filename):
    input_file = check_and_open(input_filename)
    header_line = input_file.readline().strip()
    if header_line != "ID     TYPE        LOCATION      STATUS    CURRENT  THRESHOLD":
        raise nagios_plugin.UnknownError("Unrecognised header line in 'hplog -t' output")
    for line in input_file.readlines():
        line = line.rstrip()
        if not line:
            continue
        temp_id = line[0:3].strip()
        temp_type = line[4:16].strip()
        temp_location = line[17:32].strip()
        temp_location = "%s (%s)" % (temp_location, temp_id)
        temp_status = line[33:41].strip()
        temp_current = line[42:51].split("/")[1].strip()
        temp_threshold = line[52:].split("/")[1].strip()
        if temp_status not in [ "Normal", "Nominal", "Absent" ]:
            raise nagios_plugin.CriticalError(
                "%s: temperature is '%s' (%s / %s)" % (temp_location, temp_status, 
                                                       temp_current, temp_threshold))
        # XXX - check temperatures make sense? (e.g. not >> threshold, not within 5C etc.)
        # XXX - can we do anything with type?  (Is "Basic Sensor" on G2, "ADM1022" on G1)

################################################################################

### Fans [hplog -f]
#           10        20        30        40        50        60        70
# 01234567890123456789012345678901234567890123456789012345678901234567890
# ID     TYPE        LOCATION      STATUS  REDUNDANT FAN SPEED 
#  1  Var. Speed   I/O Zone        Normal     Yes     Medium ( 45)
#  2  Var. Speed   I/O Zone        Normal     Yes     Medium ( 45)
#  3  Var. Speed   Processor Zone  Normal     Yes     Medium ( 41)
#  4  Var. Speed   Processor Zone  Normal     Yes     Low    ( 36)
#  5  Var. Speed   Processor Zone  Normal     Yes     Low    ( 36)
#  6  Var. Speed   Processor Zone  Normal     Yes     Low    ( 36)
#
def check_fans(input_filename):
    input_file = check_and_open(input_filename)
    header_line = input_file.readline().strip()
    if header_line != "ID     TYPE        LOCATION      STATUS  REDUNDANT FAN SPEED":
        raise nagios_plugin.UnknownError("Unrecognised header line in 'hplog -f' output")
    for line in input_file.readlines():
        line = line.rstrip()
        if not line:
            continue
        fan_id = line[0:3].strip()
        fan_type = line[4:16].strip()
        fan_location = line[17:32].strip()
        fan_location = "%s (%s)" % (fan_location, fan_id)
        fan_status = line[33:40].strip()
        fan_redundant = line[41:50].strip()
        fan_speed = line[51:].strip()
        if fan_type == "Basic Fan":
            continue
        if fan_type not in [ "Var. Speed", "Pwr. Supply" ]:
            raise nagios_plugin.UnknownError(
                "%s: Unrecognised fan type '%s'" % (fan_location, fan_type))
        if fan_status not in [ "Normal", "Nominal" ]:
            raise nagios_plugin.CriticalError(
                "%s: fans are '%s' (%s / Redundant: %s)" % (fan_location, fan_status, 
                                                            fan_speed, fan_redundant))
        if fan_redundant != "Yes" and fan_type == "Var. Speed":
            raise nagios_plugin.CriticalError(
                "%s: fans are not redundant (%s / Status: %s)" % (fan_location,
                                                                  fan_speed, fan_redundant))
        # XXX - warn if high speed?

################################################################################

### Power supplies [hplog -p]
#           10        20        30        40        50        60        70
# 01234567890123456789012345678901234567890123456789012345678901234567890
# ID     TYPE        LOCATION      STATUS  REDUNDANT 
#  1  Standard     Pwr. Supply Bay Normal     Yes      
#  2  Standard     Pwr. Supply Bay Normal     Yes      
def check_power(input_filename, single_psu=False):
    input_file = check_and_open(input_filename)
    header_line = input_file.readline().strip()
    if header_line != "ID     TYPE        LOCATION      STATUS  REDUNDANT":
        raise nagios_plugin.UnknownError("Unrecognised header line in 'hplog -f' output")
    for line in input_file.readlines():
        line = line.rstrip()
        if not line:
            continue
        power_id = line[0:3].strip()
        power_type = line[4:16].strip()
        power_location = line[17:32].strip()
        power_location = "%s (%s)" % (power_location, power_id)
        power_status = line[33:40].strip()
        power_redundant = line[41:50].strip()
        if power_type != "Standard":
            raise nagios_plugin.UnknownError(
                "%s: Unrecognised power type '%s'" % (power_location, power_type))
        if not single_psu and power_status not in [ "Normal", "Nominal" ]:
            raise nagios_plugin.CriticalError(
                "%s: power supply is '%s'" % (power_location, power_status))

###########################################################################
### RAID arrays [hpacucli]
##
# logicaldrive 1 (68.3 GB, RAID RAID 1+0):  OK
# logicaldrive 1 (2.8 TB, RAID RAID 6 (ADG)):  OK
# physicaldrive 1:1 (box 1:bay 1, 72 GB): OK
# physicaldrive 1:2 (box 1:bay 2, 72 GB): OK
def check_array(input_filename):
    input_file = check_and_open(input_filename)
    innocuous_errors = re.compile(r'^Error: The specified (device|controller) does not have any (logical|physical)')
    for line in input_file.readlines():
        line = line.strip()
        if not line or innocuous_errors.search(line):
            continue
        (drivetype, number) = line.split()[:2]
        status = line.split("):")[1].lstrip().upper()
        if status != "OK":
            raise nagios_plugin.CriticalError("%s %s is '%s'" % (drivetype, number, status))

###########################################################################
### RAID controllers [hpacucli]
##
# Smart Array 6400 in Slot 1
#   Controller Status: OK
#   Cache Status: OK
#   Battery Status: Failed (Replace Batteries)
def check_controller(input_filename):
    input_file = check_and_open(input_filename)
    for line in input_file.readlines():
        line = line.strip()
        if not line:
            continue
        if line.find(":") == -1:
            if line.startswith("Smart Array"):
                # [xxx could grab slot number and array type here if we cared]
                continue
            else:
                raise nagios_plugin.UnknownError("Unrecognised line for controller '%s'" % (line))
        else:
            (part, status) = line.split(":")
            status = status.strip().upper()
            if status != "OK":
                raise nagios_plugin.CriticalError("%s is '%s'" % (part, status))

################################################################################
### General log entries
##
# ID   Severity       Initial Time      Update Time       Count
# 012345678901234567890123456789012345678901234567890123456
# -------------------------------------------------------------
# 0001 Repaired       00:00  00/00/0000 05:16  04/23/2008 0003
# LOG: System Power Supply: General Failure (Power Supply 1)
#
# 0002 Caution        00:00  03/27/2008 00:00  03/27/2008 0001
# LOG: POST Error: 1615-Power Supply Failure or Power Supply Unplugged in Bay 1

def check_log(input_filename):
    try:
        uptime_file = open("/proc/uptime")
        uptime = uptime_file.read().split()[0]
        uptime_file.close()
    except:
        raise nagios_plugin.UnknownError("Cannot open /proc/uptime")

    boot_time = datetime.datetime.now() - datetime.timedelta(seconds=float(uptime))

    input_file = check_and_open(input_filename)
    input_file.readline()
    header_line = input_file.readline().strip()
    if header_line != "ID   Severity       Initial Time      Update Time       Count":
        raise nagios_plugin.UnknownError("Unrecognised header line in 'hplog -v' output")
    for line in input_file.readlines():
        line = line.rstrip()
        if not line:
            continue
        entry_id = line[0:4].strip()
        if entry_id == "----":
            continue
        elif entry_id == "LOG:":
            if entry_severity == "Repaired":
                continue
            if entry_timestamp < boot_time:
                continue
            entry_message = line[5:].strip()
            raise nagios_plugin.CriticalError("'hplog -v': %s" % entry_message)
        else:
            entry_severity = line[5:19].strip()
            entry_time = line[38:44].strip().split(":")
            entry_date = line[45:55].strip().split("/")
            # Some log entries have a date of '00/00/0000', which we have to ignore
            if entry_date[2] == '0000':
                entry_date = ('01','01','1970')
            entry_timestamp = datetime.datetime(
                int(entry_date[2]),
                int(entry_date[0]),
                int(entry_date[1]),
                int(entry_time[0]),
                int(entry_time[1]))



################################################################################

def main():
    single_psu = "-s" in sys.argv

    nagios_plugin.try_check(check_array, "/var/lib/nagios/cat-hp-array.txt")
    nagios_plugin.try_check(check_power, "/var/lib/nagios/cat-hp-power.txt", single_psu)
    nagios_plugin.try_check(check_temperature, "/var/lib/nagios/cat-hp-temperature.txt")
    nagios_plugin.try_check(check_fans, "/var/lib/nagios/cat-hp-fans.txt")
    nagios_plugin.try_check(check_controller, "/var/lib/nagios/cat-hp-controller.txt")
    nagios_plugin.try_check(check_log, "/var/lib/nagios/cat-hp-log.txt")
    print "All OK"

if __name__ == "__main__":
    main()
