#!/bin/bash

set -e
set -u

VERBOSE="0"
DAYSWARN="14"
DAYSFAIL="7"

FAIL="0"
WARN="0"
FAILHOSTS=""
FAILTXT=""
WARNHOSTS=""
WARNTXT=""
OUTPUT=""
CERT=""
FAILCERTS=""
CERTFAILS=""
VALIDITY=""

SSLHOSTS="/etc/nagios3/sslhosts.txt"

if [ ! -f $SSLHOSTS ]; then
  echo "FAIL: Unable to read $SSLHOSTS"
  exit 2
fi

if [ "$#" -eq 1 ]; then
  if [ "$1" == "--verbose" ]; then
    VERBOSE="1"
    echo "Enabling verbose mode"
  fi
fi

for sslhost in $(grep -v ^# $SSLHOSTS) ; do
  if [ "$VERBOSE" == "1" ]; then
    echo -n "Checking $sslhost..."
  fi
  if [ "${sslhost:0:1}" == "/" ]; then
    # This is a local certificate file
    if [ "${sslhost: -4:4}" == ".spc" ]; then
      # Convert .spc to x509
      SSLOUTPUT=$(openssl pkcs7 -inform DER -print_certs -in ${sslhost})
    else
      # Assume file is already x509
      SSLOUTPUT=$(cat ${sslhost})
    fi
  else
    # This is a remote host:port pair
    set +e
    SSLOUTPUT=$(echo "" | openssl s_client -tls1 -CApath /etc/ssl/certs/ -servername ${sslhost/:*/} -connect $sslhost 2>&1)
    if [ $? -ne 0 ]; then
      FAIL="1"
      FAILTXT="${FAILTXT} 
      SERIOUS: ${sslhost} connection failed"
      FAILHOSTS="C:${sslhost} ${FAILHOSTS}"
      continue
    fi
    set -e
  fi

  CERT=$(echo "$SSLOUTPUT" | sed -n '/-----BEGIN CERTIFICATE-----/,/-----END CERTIFICATE-----/p')
  EXPIRY=$(echo "$CERT" | openssl x509 -enddate | awk -F= ' /notAfter/ { printf("%s\n",$NF); } ')

  set +e
  VALIDITY=$(echo "$SSLOUTPUT" | grep "Verify return code: " | grep -v "Verify return code: 0 (ok)")
  set -e

  EXPIRDATE=$(date -d "$EXPIRY" +"%s")
  NOWDATE=$(date +"%s")
  DATEDIFF=$(( $EXPIRDATE - $NOWDATE ))
  DAYSDIFF=$(( $DATEDIFF / 86400 ))

  if [ "$VALIDITY" != "" ]; then
    ERROR=$(echo $VALIDITY | cut -f 4 -d ' ')
    case $ERROR in
      18) # Self-signed certificate
        ;;
      *)
        FAIL="1"
        CERTFAILS="${CERTFAILS}
        ${sslhost} verify error $ERROR"
        FAILCERTS="V:${sslhost} ${FAILCERTS}"
        ;;
    esac
  fi

  if [ "$DAYSDIFF" -lt "$DAYSFAIL" ]; then
    FAIL="1"
    FAILTXT="${FAILTXT} 
    SERIOUS: ${sslhost} expires in ${DAYSDIFF} days"
    FAILHOSTS="${FAILHOSTS}E:$(echo ${sslhost} | sed -re 's/:[0-9]+//'):${DAYSDIFF}d "
  elif [ "$DAYSDIFF" -lt "$DAYSWARN" ]; then
    WARN="1"
    WARNTXT="${WARNTXT} 
    WARNING: ${sslhost} expires in ${DAYSDIFF} days"
    WARNHOSTS="${WARNHOSTS}E:$(echo ${sslhost} | sed -re 's/:[0-9]+//'):${DAYSDIFF}d "
  fi

  if [ "$VERBOSE" == "1" ]; then
    echo "Done. ${DAYSDIFF} days remain."
  fi
done

if [ "$FAIL" == "1" ]; then
  OUTPUT="FAIL: "
  OUTPUT="${OUTPUT} ${FAILHOSTS} ${FAILCERTS}"
  if [ "$WARNHOSTS" != "" ]; then
    OUTPUT="${OUTPUT} WARN: ${WARNHOSTS}"
  fi
fi

if [ "$WARN" == "1" -a "$OUTPUT" == "" ]; then 
  OUTPUT="WARN: ${WARNHOSTS}"
fi

if [ "$OUTPUT" != "" ]; then
  echo $OUTPUT
  if [ "$VERBOSE" == "1" ]; then
    echo "$FAILTXT $CERTFAILS $WARNTXT"
    echo ""
  fi
  if [ "$FAIL" == "1" ]; then
    exit 2
  fi
  if [ "$WARN" == "1" ]; then
    exit 1
  fi
else
  echo "OK: All certs ok"
fi
